/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.SoilType;
import nl.cloudfarming.client.util.swing.FormRow;
import nl.cloudfarming.client.util.swing.beanbinding.NonEmptyStringValidator;
import nl.cloudfarming.client.util.swing.beanbinding.ValidityDateValidator;
import org.jdesktop.beansbinding.*;
import org.jdesktop.swingx.JXDatePicker;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXTextField;
import org.jdesktop.swingx.combobox.EnumComboBoxModel;
import org.jdesktop.swingx.combobox.ListComboBoxModel;
import org.openide.util.NbBundle;

/**
 * Panel which displays the data of a field
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({"FieldDataPanel unit abbreviation hectares=ha",
    "FieldDataPanel.nameLabel.text=Name:",
    "FieldDataPanel.validityLabel.text=Validity:",
    "FieldDataPanel.areaLabel.text=Area:",
    "FieldDataPanel.areaField.text=0.0 ha",
    "FieldDataPanel.soilTypeLabel.text=Soil type:"})
public class FieldDataPanel extends javax.swing.JPanel {

    private BeanProperty nameTextField = BeanProperty.create("text");
    private BeanProperty fieldPropName = BeanProperty.create(Field.PROP_NAME);
    private Binding nameBinding;
    private BeanProperty textProperty = BeanProperty.create("text");
    private BeanProperty fieldPropArea = BeanProperty.create(Field.PROP_AREA_IN_HECTARES);
    private Binding areaBinding;
    private BeanProperty formPropValidityStart = BeanProperty.create("date");
    private BeanProperty fieldPropStartDate = BeanProperty.create(Field.PROP_START_DATE);
    private Binding validityStartBinding;
    private BeanProperty formPropValidityEnd = BeanProperty.create("date");
    private BeanProperty fieldPropEndDate = BeanProperty.create(Field.PROP_END_DATE);
    private Binding validityEndBinding;
    private BeanProperty formPropSoilType = BeanProperty.create("selectedItem");
    private BeanProperty fieldPropSoilType = BeanProperty.create(Field.PROP_SOIL_TYPE);
    private Binding soilTypeBinding;
    private BindingGroup fieldBindingGroup = new BindingGroup();
    private final Field field;
    private JLabel areaField;
    private JTextField nameField;
    private JXDatePicker validityEndField;
    private JXDatePicker validityStartField;
    private JComboBox soilTypeComboBox;
    private FormRow nameRow;
    private FormRow validityRow;
    private FormRow areaRow;
    private FormRow soilTypeRow;
    public static final String START_DATE_PICKER_ACTION_COMMAND = "startdateChanged";
    public static final String END_DATE_PICKER_ACTION_COMMAND = "enddateChanged";

    /**
     * Creates new form FieldDataPanel
     */
    public FieldDataPanel(Field field) {
        assert field != null : "Cannot construct a FieldDataPanel with a null field";
        this.field = field;
        initValues();
        bind();
    }

    /**
     * Initiate field values to values in the Field object
     */
    private void initValues() {
        setLayout(new MigLayout("wrap 1"));
        nameField = new JXTextField();
        nameField.setText(field.getName());
        nameRow = new FormRow(true, Bundle.FieldDataPanel_nameLabel_text(), Bundle.FieldDataPanel_nameLabel_text(), nameField);
        validityStartField = new JXDatePicker();

        validityEndField = new JXDatePicker();

        validityRow = new FormRow(true, Bundle.FieldDataPanel_validityLabel_text(), Bundle.FieldDataPanel_validityLabel_text(), validityStartField, validityEndField);

        areaField = new HectaresLabel();
        areaField.setText(String.valueOf(field.getAreaInHectares()));
        areaRow = new FormRow(false, Bundle.FieldDataPanel_areaLabel_text(), Bundle.FieldDataPanel_areaLabel_text(), areaField);

        soilTypeComboBox = new JComboBox();
        ListComboBoxModel<SoilType> soilTypeModel = new EnumComboBoxModel<>(SoilType.class);
        soilTypeComboBox.setModel(soilTypeModel);
        soilTypeModel.setSelectedItem(field.getSoilType());
        soilTypeRow = new FormRow(false, Bundle.FieldDataPanel_soilTypeLabel_text(), Bundle.FieldDataPanel_soilTypeLabel_text(), soilTypeComboBox);

        this.add(nameRow);
        this.add(validityRow);
        this.add(areaRow);
        this.add(soilTypeRow);
    }

    /**
     * bind form fields to Field properties
     */
    private void bind() {
        nameBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropName, nameField, nameTextField);
        nameBinding.setValidator(new NonEmptyStringValidator());
        fieldBindingGroup.addBinding(nameBinding);

        areaBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ,
                field, fieldPropArea, areaField, textProperty);
        fieldBindingGroup.addBinding(areaBinding);


        validityStartBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropStartDate, validityStartField, formPropValidityStart);

        fieldBindingGroup.addBinding(validityStartBinding);

        validityEndBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropEndDate, validityEndField, formPropValidityEnd);

        fieldBindingGroup.addBinding(validityEndBinding);

        ValidityDateValidator dateValidator = new ValidityDateValidator(validityStartBinding, validityEndBinding, true);
        validityStartBinding.setValidator(dateValidator);
        validityEndBinding.setValidator(dateValidator);

        soilTypeBinding = Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ_WRITE,
                field, fieldPropSoilType, soilTypeComboBox, formPropSoilType);
        fieldBindingGroup.addBinding(soilTypeBinding);
        fieldBindingGroup.bind();
    }

    /**
     * Get the name field from this panel
     *
     * @return
     */
    public JTextField getNameField() {
        return nameField;
    }

    /**
     * Get the (@link FormRow) which holds the name field
     *
     * @return
     */
    public FormRow getNameRow() {
        return nameRow;
    }

    /**
     * Get the validity end field from this panel
     *
     * @return
     */
    public JXDatePicker getValidityEndField() {
        return validityEndField;
    }

    /**
     * Get the validity start field from this panel
     *
     * @return
     */
    public JXDatePicker getValidityStartField() {
        return validityStartField;
    }

    /**
     * Get the {@link FormRow} which holds the start and end date
     *
     * @return
     */
    public FormRow getValidityRow() {
        return validityRow;
    }

    /**
     * Get the surface field from this panel
     *
     * @return
     */
    public JLabel getAreaField() {
        return areaField;
    }

    /**
     * Get the soil type combobox from this panel
     *
     * @return
     */
    public JComboBox getSoilTypeComboBox() {
        return soilTypeComboBox;
    }

    public class HectaresLabel extends JXLabel {

        @Override
        public void setText(String text) {
            super.setText(text + " " + Bundle.FieldDataPanel_unit_abbreviation_hectares());
        }
    }
}
