/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.type.wizard.NewActivityFieldAction;
import nl.cloudfarming.client.sensor.ObservationNodeProvider;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.datatransfer.PasteType;

/**
 * Node for {@link Field}
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
final class FieldNode extends FilterNode {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field.type.FieldNode");
    private final Field field;
    //Property to indicate that the changes are incomming from the user side not the model
    //this prevents the setName method from being called twice after changing the name
    //(because the FieldNode listens to the Field and changing the name would trigger
    //a PropertyChangeEvent that would trigger setName again)
    private boolean userChange = true;

    /**
     * Constructs a new FieldNode
     *
     * @param oFieldNode the original FieldNode
     * @param oFieldFolderNode the original FolderNode
     * @param field the Field for which this node is
     */
    public FieldNode(Node oFieldNode, Field field) {
        super(oFieldNode,
                Children.create(new FieldNodeChildFactory(field), true));


        LOGGER.log(Level.FINEST, "created new field node with field {0}", System.identityHashCode(field));
        this.field = field;
        disableDelegation(DELEGATE_SET_NAME);
        disableDelegation(DELEGATE_GET_NAME);
        setName(field.getName());
        setDisplayName(getName());

        field.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (Field.PROP_NAME.equals(evt.getPropertyName())) {
                    userChange = false;
                    setName((String) evt.getNewValue());
                    setDisplayName(getName());
                    userChange = true;
                }
            }
        });
    }

    /**
     * Override the method to set the name on the field which this node is based
     * on
     *
     * @param name
     */
    @Override
    public void setName(String name) {
        super.setName(name);
        if (userChange) {
            field.setName(name);
        }
    }

    @Override
    public PasteType getDropType(Transferable t, int action, int index) {
        // rely on AbstractNode behavior which checks if the transferable contains a NodeTransfer.Paste.
        return super.getDropType(t, action, index);
    }

    @Override
    public Action[] getActions(boolean context) {
        List<Action> actions = new ArrayList(Arrays.asList(super.getActions(context)));
        actions.add(new NewActivityFieldAction(field));
        return actions.toArray(new Action[actions.size()]);
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        Transferable deflt = NodeTransfer.transferable(this, NodeTransfer.CLIPBOARD_COPY);
        ExTransferable added = ExTransferable.create(deflt);
        added.put(new ExTransferable.Single(Field.DATA_FLAVOR) {

            @Override
            protected Field getData() {
                return field;
            }
        });
        return added;
    }

    @Override
    public Transferable clipboardCut() throws IOException {
        return clipboardCopy();
    }

    @Override
    public Transferable drag() throws IOException {
        return clipboardCopy();
    }

    /**
     * Child factory for FieldNode
     */
    private static final class FieldNodeChildFactory extends ChildFactory<Node> {

        private final Field field;

        public FieldNodeChildFactory(Field field) {
            this.field = field;
            field.addPropertyChangeListener(new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    if (Field.PROP_ACTIVITYFIELDS.equals(evt.getPropertyName())) {
                        refresh(true);
                    }
                }
            });
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            for (ActivityField activityField : field.getActivityFields()) {
                toPopulate.add(new ActivityFieldNode(activityField));
            }
            toPopulate.add(ObservationNodeProvider.getObservationsNode(field));
            return true;
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }
    }
}
